package com.isu.gaswellwatch.modbus.data.listener;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/23 0:32
 */

import cn.hutool.core.map.MapUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;

import java.util.Map;


@Component
public class DynamicRabbitListener implements ApplicationRunner {

    private static final String DEVICE_SQL = "SELECT d.id, d.gateway_sn AS identifier FROM device d where d.id = ";
    private static final String GATEWAY_SQL = "SELECT d.id, d.gateway_sn AS identifier FROM device d ORDER BY d.id ";

    private final AmqpAdmin amqpAdmin;
    private final JdbcTemplate jdbcTemplate;
    private final ComposeModbusMessageListener composeListener;
    private final SimpleMessageListenerContainer modbusMessageListenerContainer;

    public DynamicRabbitListener(ConnectionFactory connectionFactory, JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
        this.amqpAdmin = new RabbitAdmin(connectionFactory);
        this.modbusMessageListenerContainer = new SimpleMessageListenerContainer();
        this.modbusMessageListenerContainer.setConnectionFactory(connectionFactory);
        this.modbusMessageListenerContainer.setMessageListener(this.composeListener = new ComposeModbusMessageListener());
        // 启动监听容器
        this.modbusMessageListenerContainer.start();
    }

    @Override
    public void run(ApplicationArguments args) {
        this.composeListener.addBatchMessageListener(new ModbusMessageBackupListener());
        this.composeListener.addBatchMessageListener(new ModbusMessagePersistListener());
        this.jdbcTemplate.queryForList(GATEWAY_SQL).forEach(this::registerDeviceAndListener);
    }

    public void registerDeviceAndListener(Long deviceId) {
        this.jdbcTemplate.queryForList(DEVICE_SQL + deviceId).forEach(this::registerDeviceAndListener);
    }

    public void addListenerCollectQueue(Queue queue, String exchangeName, String routingKey) {
        if (StringUtils.isNotBlank(exchangeName) && StringUtils.isNotBlank(routingKey)) {
            // 声明直接类型的交换器
            DirectExchange exchange = new DirectExchange(exchangeName);
            this.amqpAdmin.declareExchange(exchange);
            // 绑定队列和交换器
            this.amqpAdmin.declareBinding(BindingBuilder.bind(queue).to(exchange).with(routingKey));
        }
        // 设置监听的队列
        this.modbusMessageListenerContainer.addQueues(queue);
    }

    private void registerDeviceAndListener(Map<String, Object> deviceMap) {
        Queue eventQueue = QueueBuilder.durable(String.format(Queues.DEVICE_EVENTS,
                MapUtil.getStr(deviceMap, "id"))).build();
        Queue collectQueue = QueueBuilder.durable(String.format(Queues.MODBUS_COLLECT_DATA,
                MapUtil.getStr(deviceMap, "identifier"))).build();
        this.amqpAdmin.declareQueue(eventQueue);
        this.amqpAdmin.declareQueue(collectQueue);
        this.addListenerCollectQueue(collectQueue, null, null);
    }

}